/* vi: set filetype=cpp ts=4 sts=4 sw=4 ai et ic: */
/**+
*******************************************************************************
*** @file           KeyPilotUSBUserDLL.h
*** PROJECT / SUB:  KeyPilotUSBDLL / USBDLL
*** DESCRIPTION:    exportierte Funktionen
*** CREATION DATE:  2009/03/06
*** AUTHOR:         Markus Biggel
*******************************************************************************
*** REVISION:       $Revision: 4071 $
*** REVISOR:        $Author: MTallowitz $
*** REVISION DATE:  $Date: 2010-12-09 13:39:49 +0100 (Do, 09 Dez 2010) $
*** IDENT:          $Id: KeyPilotUSBUserDLL.h 4071 2010-12-09 12:39:49Z MTallowitz $
*******************************************************************************
*** @brief          Definitionen & Funktionsprototypen fuer den Zugriff auf
***                   KeyPilots/USB.
*******************************************************************************
-**/

#if defined DOXYGEN_WITH_EXAMPLES
/// \mainpage KeyPilot USB DLL Dokumentation
/// Dieses Dokument enth&auml;lt die Dokumentation der KeyPilot USB DLL.
/// Ausserdem ist die Dokumentation der mitgelieferten Programmierbeispielen f&uuml;r
/// die Verwendung der DLL aus unterschiedlichen Programmiersprachen heraus enthalten.
/// Die eigentliche DLL wird unter KeyPilotUSBUserDLL.h in der Datei&uuml;bersicht dokumentiert.
#elif defined DOXYGEN_USER
/// \mainpage KeyPilot USB DLL Dokumentation
/// Dieses Dokument enth&auml;lt die Dokumentation der KeyPilot USB DLL.
#endif

#ifndef __KP_USBDLL_KEYPILOT_USER_DLL_H__
#define __KP_USBDLL_KEYPILOT_USER_DLL_H__

#ifndef DOXYGEN
#    include <windows.h>
#endif /* !DOXYGEN */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#ifndef DOXYGEN
#    ifndef DECLSPEC
#        define DECLSPEC __declspec( dllimport )
#    endif /* !DECLSPEC */
#endif /* !DOXYGEN */

/*! \name String Gr&ouml;&szlig;en
@{
*/
/// \brief Mindestgr&ouml;&szlig;e des Buffers, in den die Firmware Version geschrieben werden soll (venwendet in kpUSBGetFirmwareVersion())
#define KP_FIRMWARE_VERSION_LENGTH   11
/// \brief Mindestgr&ouml;&szlig;e des Buffers, in den die Seriennummer geschrieben werden soll (verwendet in kpUSBGetSerialNumber())
#define KP_SERIAL_NUMBER_LENGTH      50
/// \brief Mindestgr&ouml;&szlig;e des Buffers, in den der Errorsting geschrieben werden soll (vewendet in kpUSBGetErrorString())
#define KP_ERRSTRING_LENGTH         100
/*! @} */

/*! \name Verbindungsstatus
@{
*/
/// \hideinitializer
/// \brief Dieser KeyPilot ist momentan nicht verwendbar, weil er beispielsweise von einem anderen Programm verwendet wird
#define  NOT_AVAILABLE              0
/// \hideinitializer
/// \brief KeyPilot ist Verf&uuml;gbar
#define  AVAILABLE                  1
/// \hideinitializer
/// \brief zu diesem KeyPilot besteht (bereits) eine offene Verbindung
#define  CONNECTED                  2
/*! @} */

/*! \name Fehlercodes
@{
*/
#define KP_OK                        0
#define KP_ERR_NO_DEVICE_FOUND      -1
#define KP_ERR_CONNECTION_FAILED    -4
#define KP_ERR_NOT_CONNECTED        -5
#define KP_ERR_WRONG_PAGE_SIZE      -6
#define KP_ERR_BUFFER_TOO_SMALL     -7
#define KP_ERR_DEVICE_BUSY          -10
#define KP_ERR_DEVICE_NOT_FOUND     -11
#define KP_ERR_UNKNOWN              -12
#define KP_ERR_ILLEGAL_DEVICE       -13
#define KP_ERR_DATASIZE_WRONG       -14
#define KP_ERR_READ_ERROR           -15
#define KP_ERR_TIMEOUT              -16
#define KP_ERR_WRONG_NR_OF_PAGES    -17
#define KP_ERR_WRONG_KEY_TYPE       -18
#define KP_ERR_IN_EVENTMODE         -19
#define KP_ERR_ILLEGAL_PAGE         -20
#define KP_ERR_WRITE_ERROR          -21
#define KP_ERR_CORRUPT_DATA         -23
#define KP_ERR_NOT_IMPLEMENTED      -24
#define KP_ERR_NO_KEY_ON_PILOT      -25
#define KP_ERR_TO_FEW_DATA          -26
#define KP_ERR_INDEX_OUT_OF_RANGE   -27
#define KP_ERR_ILLEGAL_COLOR        -28
#define KP_ERR_NULLPOINTER          -29
/*! @} */

// ----------------- Erklaerungen der Fehlercodes
/// \def KP_OK
/// \brief Allgemeine Erfolgsmeldung

/// \def KP_ERR_NO_DEVICE_FOUND
/// \brief Es wurde kein KeyPilot gefunden

/// \def KP_ERR_CONNECTION_FAILED
/// \brief es konnte keine Verbindung hergestellt werden

/// \def KP_ERR_NOT_CONNECTED
/// \brief es besteht keine Verbindung

/// \def KP_ERR_WRONG_PAGE_SIZE
/// \brief Die angegebene Pagegr&ouml;&szlig;e ist falsch

/// \def KP_ERR_BUFFER_TOO_SMALL
/// \brief der &uuml;bergebene Buffer ist zu klein

/// \def KP_ERR_DEVICE_BUSY
/// \brief das Ger&auml;t ist besch&auml;ftigt, z.B. weil ein anderer Prozess eine Verbindung offen hat

/// \def KP_ERR_DEVICE_NOT_FOUND
/// \brief Das angegebene Ger&auml;t konnte nicht gefunden werden

/// \def KP_ERR_UNKNOWN
/// \brief ein unbekannter oder nicht weiter spezifizierter Fehler ist aufgetreten
 
/// \def KP_ERR_ILLEGAL_DEVICE
/// \brief Falsche Angaben zum Ger&auml;t

/// \def KP_ERR_DATASIZE_WRONG
/// \brief ein Datensatz hat die falsche Gr&ouml;sse

/// \def KP_ERR_READ_ERROR
/// \brief beim Lesen ist ein Fehler aufgetreten

/// \def KP_ERR_TIMEOUT
/// \brief ein Timeout ist aufgetreten

/// \def KP_ERR_IN_EVENTMODE
/// \brief Fehler wird zur&uuml;ckgegeben, wenn ein Ausleseversuch stattfindet solange sich der KeyPilot im Eventmode befindet

/// \def KP_ERR_WRONG_NR_OF_PAGES 
/// \brief Die Anzahl der Pages ist ung&uuml;ltig

/// \def KP_ERR_WRONG_KEY_TYPE 
/// \brief Der aufgelegte Schl&uuml;ssel ist vom falschen Typ

/// \def KP_ERR_CORRUPT_DATA
/// \brief Fehlerhafte Daten vom KeyPilot empfangen

/// \def KP_ERR_NO_KEY_ON_PILOT
/// \brief Auf dem KeyPilot liegt kein Schl&uuml;ssel auf
  
/// \def KP_ERR_TO_FEW_DATA
/// \brief Der &uuml;bergebene Datensatz ist kleiner als die Anzahl der zu schreibenden Seiten multipliziert mit der Pagegr&ouml;&szlig;e.

/// \def KP_ERR_INDEX_OUT_OF_RANGE
/// \brief Fehlercode wenn der gew&auml;hlte Index gr&ouml;&szlig;er ist als die Anzahl der KeyPilots

/// \def KP_ERR_ILLEGAL_PAGE
/// \brief Eine ung&uuml;ltige Seite wurde ausgewhlt

/// \def KP_ERR_WRITE_ERROR
/// \brief Ein Fehler beim Schreiben ist aufgetreten

/// \def KP_ERR_NOT_IMPLEMENTED
/// \brief Diese Funktion ist noch nicht implementiert

/// \def KP_ERR_ILLEGAL_COLOR
/// \brief Eine ung&uuml;ltige Farbe wurde angegeben

/// \def KP_ERR_NULLPOINTER
/// \brief Ein NULL Pointer wurde entdeckt wo keiner sein sollte


/*! \name LED Farbcodes
@{
*/
#define KP_LED_COLOR_GREEN          0
#define KP_LED_COLOR_RED            1
#define KP_LED_COLOR_OFF            2
/*! @} */

/// \brief  Schl&uuml;sseldaten
typedef struct _KEYPILOT_KEYDATA {
   /// \brief 1 wenn die Daten G&uuml;ltig sind, 0 wenn sie ung&uuml;ltig sind, z.B. wenn kein Schl&uuml;ssel aufliegt
   int      validData;
   /// \brief das abgesetzte Kommando
   char     command[12];
   /// \brief ID des aufliegenden Schl&uuml;ssels
   char     keyID[13];
   /// \brief Family Code des aufliegenden Schl&uuml;ssels
   char     famCode[3];
   /// \brief CRC der Schl&uuml;sseldaten
   char     crc[3];
} KEYPILOT_KEYDATA, *PKEYPILOT_KEYDATA, *PKP_KEYDATA, KP_KEYDATA;

#ifdef DOXYGEN
typedef KP_KEYDATA _KEYPILOT_KEYDATA;
typedef KP_KEYDATA KEYPILOT_KEYDATA;
typedef KP_KEYDATA *PKEYPILOT_KEYDATA;
typedef KP_KEYDATA *PKP_KEYDATA;
#endif

/// \typedef KEYPILOT_KEYDATA 
/// \deprecated benutze stattdessen \ref KP_KEYDATA
/// \brief  Schl&uuml;sseldaten (Alias f&uuml;r \ref _KEYPILOT_KEYDATA)

/// \typedef KP_KEYDATA 
/// \brief  Schl&uuml;sseldaten (Alias f&uuml;r \ref _KEYPILOT_KEYDATA)

/// \typedef PKEYPILOT_KEYDATA
/// \deprecated benutze stattdessen \ref PKP_KEYDATA
/// \brief Pointer auf \ref KEYPILOT_KEYDATA bzw \ref _KEYPILOT_KEYDATA bzw \ref KP_KEYDATA (=\ref PKP_KEYDATA)

/// \typedef PKP_KEYDATA
/// \brief Pointer auf \ref KP_KEYDATA bzw \ref _KEYPILOT_KEYDATA bzw \ref KEYPILOT_KEYDATA (=\ref PKP_KEYDATA)

/// \brief  Plattformunabh&auml;ngige Informationen zum USB KeyPilot
typedef struct _KEYPILOT_USB_DEVICE {
   /// \brief Seriennummer des KeyPilot
   char              serialNumber[50];
   /// \brief Gibt den Status der Aktuellen Verbindung an (\ref AVAILABLE, \ref CONNECTED oder \ref NOT_AVAILABLE)
   int               connectionStatus;
   /// \brief 1 = Event Mode An, 0 = Event Mode Aus
   int               eventMode;
   /// \brief Enth&auml;lt den Namen des Herstellers
   char              vendorName[100];
   /// \brief enth&auml;lt die Identifikationsnummer des Herstellers
   int               vendorID;
   /// \brief enth&auml;lt die Identifikationsnummer des Produktes
   int               productID;
   /// \brief enth&auml;lt die Version der Hardware
   int               version;
   /// \brief enth&auml;lt irgendeinen Namen
   char              friendlyName[255];
   /// \brief enth&auml;lt eine Beschreibung des Ger&auml;tes
   char              description[255];
   /// \brief enth&auml;lt den Produktnamen
   wchar_t           prName[128];
   /// \brief enth&auml;lt die Identifikationsnummer des Ger&auml;tes
   int               deviceID;
} KEYPILOT_USB_DEVICE, *PKEYPILOT_USB_DEVICE, *PKP_USB_DEV, KP_USB_DEV;

#ifdef DOXYGEN
typedef KP_USB_DEV _KEYPILOT_USB_DEVICE;
typedef KP_USB_DEV KEYPILOT_USB_DEVICE;
typedef KP_USB_DEV *PKEYPILOT_USB_DEVICE;
typedef KP_USB_DEV *PKP_USB_DEV;
#endif

/// \typedef KEYPILOT_USB_DEVICE
/// \deprecated benutze stattdessen KP_USB_DEV
/// \brief  Plattformunabh&auml;ngige Informationen zum USB KeyPilot (Alias f&uuml;r \ref _KEYPILOT_USB_DEVICE)

/// \typedef KP_USB_DEV
/// \brief  Plattformunabh&auml;ngige Informationen zum USB KeyPilot (Alias f&uuml;r \ref _KEYPILOT_USB_DEVICE)

/// \typedef PKEYPILOT_USB_DEVICE
/// \deprecated benutze stattdessen \ref PKP_USB_DEV
/// \brief Pointer auf \ref KEYPILOT_USB_DEVICE bzw \ref _KEYPILOT_USB_DEVICE bzw \ref KP_USB_DEV (=\ref PKP_USB_DEV)

/// \typedef PKP_USB_DEV
/// \brief Pointer auf \ref KP_USB_DEV bzw \ref KEYPILOT_USB_DEVICE bzw \ref _KEYPILOT_USB_DEVICE (=\ref PKEYPILOT_USB_DEVICE)

/// \brief Verkettete Liste mit KeyPilots
typedef struct _KEYPILOT_USB_DEVICELIST {
   KP_USB_DEV*      device;
   struct _KEYPILOT_USB_DEVICELIST* next;
} KEYPILOT_USB_DEVICELIST, *PKEYPILOT_USB_DEVICELIST, *PKP_USB_DEVLIST, KP_USB_DEVLIST;

#ifdef DOXYGEN
typedef KP_USB_DEVLIST KEYPILOT_USB_DEVICELIST;
typedef KP_USB_DEVLIST *PKEYPILOT_USB_DEVICELIST;
typedef KP_USB_DEVLIST *PKP_USB_DEVLIST;
typedef KP_USB_DEVLIST _KEYPILOT_USB_DEVICELIST;
#endif

/// \typedef KEYPILOT_USB_DEVICELIST
/// \deprecated benutze stattdessen \ref KP_USB_DEVLIST
/// \brief Liste mit KeyPilots (Alias f&uuml;r \ref _KEYPILOT_USB_DEVICELIST)

/// \typedef KP_USB_DEVLIST
/// \brief Liste mit KeyPilots (Alias f&uuml;r \ref _KEYPILOT_USB_DEVICELIST)

/// \var KP_USB_DEV* _KEYPILOT_USB_DEVICELIST::device
/// \brief Ein KeyPilot

/// \var struct _KEYPILOT_USB_DEVICELIST* _KEYPILOT_USB_DEVICELIST::next
/// \brief Das n&auml;chste Element in der Liste

/// \typedef PKEYPILOT_USB_DEVICELIST
/// \deprecated benutze stattdessen \ref PKP_USB_DEVLIST
/// \brief Pointer auf \ref KEYPILOT_USB_DEVICELIST bzw \ref _KEYPILOT_USB_DEVICELIST bzw \ref KP_USB_DEVLIST (=\ref PKP_USB_DEVLIST)

/// \typedef PKP_USB_DEVLIST
/// \brief Pointer auf \ref KP_USB_DEVLIST bzw \ref KEYPILOT_USB_DEVICELIST bzw \ref _KEYPILOT_USB_DEVICELIST (=\ref PKEYPILOT_USB_DEVICELIST)


/*! \name Funktionspointer f&uuml;r die CALLBACK Funktionen
@{
*/
#ifndef DOXYGEN
typedef  void  (WINAPI *pfCallbackKeyNotice) (int deviceID_, KP_KEYDATA* kpData_ );
typedef  void  (WINAPI *pfCallbackDisconnectNotice) ( int deviceID_ );
#else
// Doxygen kommt mit WINAPI nicht klar, deshalb weglassen
// TODO: WINAPI als Text dokumentieren
typedef  void  (*pfCallbackKeyNotice) (int deviceID_, KP_KEYDATA* kpData_ );
typedef  void  (*pfCallbackDisconnectNotice) ( int deviceID_ );
#endif
/*! @} */

/// \typedef    pfCallbackKeyNotice
/// \brief      Funktionspointer fuer Event Mode-Callback Funktion die aufgerufen wird, wenn Daten am KeyPilot ankommen.
/// \details    Funktionen, deren Funktionspointer als Notice-Callback an die Methode 
///             #kpUSBSetEventmode &uuml;bergeben werden sollen, m&uuml;ssen diese Signatur vorweisen:<br>
///             <code>void NoticeCallback( int deviceID_, KP_KEYDATA* kpData_ )</code><br>
///             wobei<br>
///             <i>deviceID_</i> ID des betreffenden KeyPilot
///             <i>kpData_</i> Struktur mit Daten des Schl&uuml;ssels der aufgelegt (oder entfernt) wurde.<br>
///             <code>typedef  void  (WINAPI *pfCallbackKeyNotice) (int deviceID_, KP_KEYDATA* kpData_ )</code>

/// \var        pfCallbackKeyNotice::deviceID_
/// \brief      ID des betreffenden KeyPilot

/// \var        pfCallbackKeyNotice::kpData_
/// \brief      Struktur mit Daten des Schl&uuml;ssels der aufgelegt (oder entfernt) wurde

/// \typedef    pfCallbackDisconnectNotice
/// \brief      Funktionspointer fuer Event Mode-Callback Funktion die aufgerufen wird, wenn die Verbindung zum KeyPilot abbricht.
/// \details    Funktionen, deren Funktionspointer als Disconnect-Callback an die Methode 
///             #kpUSBSetEventmode &uuml;bergeben werden sollen, m&uuml;ssen diese Signatur vorweisen:<br>
///             <code>void DisconnectCallback( int deviceID_ )</code><br>
///             wobei<br>
///             <i>deviceID_</i> ID des betreffenden KeyPilot<br>
///             <code>typedef  void  (WINAPI *pfCallbackDisconnectNotice) ( int deviceID_ )</code>

/// \var        pfCallbackDisconnectNotice::deviceID_
/// \brief      ID des betreffenden KeyPilot

/// \brief       Aktualisiert die KeyPilot Liste. Der HID Stack wird nach KeyPilots abgesucht. Wurde
///              ein bisher unbekannter KeyPilot gefunden, wird dieser in die Liste aufgenommen, bekannte 
///              Ger&auml;te werden aktuallisiert und entfernte Ger&auml;te werden entfernt.
/// \return      ::KP_OK bei Erfolg
DECLSPEC int WINAPI kpUSBScanDevices();

/// \brief        Speichert die vorhandenen KeyPilots in das mitgelieferte Array, dieses muss
///               bereits allokiert sein. Das Array muss mindestens soviele elemente besitzen
///               wie KeyPilots vorhanden sind. Die Anzahl der vorhandenen Ger&auml;te kann mit
///               kpUSBGetNumberOfDevices festgestellt werden.
/// \param[out]   kps_ Array mit allen angeschlossenen KeyPilots (muss allokiert sein)
/// \param[in]    length_ Gr&ouml;&szlig;e des Arrays (mindestens die Gr&ouml;&szlig;e die kpUSBGetNumberOfDevices() liefert)
/// \return       ::KP_OK bei Erfolg<br>
///               ::KP_ERR_BUFFER_TOO_SMALL wenn der angegebene Puffer zu klein ist
DECLSPEC int WINAPI kpUSBGetDeviceList(
                        KP_USB_DEV kps_[],
                        int length_ );

/// \brief           Liefert einen bestimmten KeyPilot aus dem Array das kpUSBGetDeviceList liefert<br>
///                  Diese Funktion wurde aus Kompatiblit&auml;tsgr&uuml;nden zu Visual Basic erstellt
/// \param[out]      kp_ KEYPILOT_USB_DEVICE Struktur des ausgew&auml;hlten KeyPilots
/// \param[in]       index_ Index des KeyPilot im Array
/// \return          ::KP_OK wenn kein Fehler aufgetreten ist<br>
///                  ::KP_ERR_INDEX_OUT_OF_RANGE wenn der Index gr&ouml;&szlig;er ist als die Anzahl der KeyPilots
DECLSPEC int WINAPI kpUSBGetDeviceListIndex(
                        KP_USB_DEV* kp_,
                        int index_ );
                        
/// \brief        Liefert die Anzahl an angeschlossenen und erreichbaren KeyPilots zur&uuml;ck
/// \return       Anzahl der gefundenen und erreichbaren (d.h. nicht durch andere Programme blockierte) KeyPilots
DECLSPEC int WINAPI kpUSBGetNumberOfDevices();

/// \brief           Mit dieser Funktion kann eine Verbindung zu einem bestimmten KeyPilot hergestellt werden
/// \param[in]       deviceID_    ID des KeyPilots zu dem verbunden werden soll
/// \return          ::KP_OK wenn Verbindung erfolgreich sonst entsprechende Fehlermeldung<br>
///                  ::KP_ERR_DEVICE_NOT_FOUND wenn das angegebene Ger&auml;t nicht gefunden wurde<br>
///                  ::KP_ERR_NO_DEVICE_FOUND wenn kein KeyPilot angeschlossen ist.<br>
///                  ::KP_ERR_CONNECTION_FAILED wenn der Verbindungsaufbau fehlgeschlagen ist
DECLSPEC int WINAPI kpUSBConnectToDevice(
                        int deviceID_ );
                        
/// \brief           Trennt die Verbindung zu einem bestimmten KeyPilot
/// \param[in]       deviceID_    ID des KeyPilots, dessen Verbindung getrennt werden soll
/// \return          ::KP_OK wenn Verbindung erfolgreich<br>
///                  ::KP_ERR_DEVICE_NOT_FOUND wenn das Ger&auml;t nicht gefunden wurde<br>
///                  ::KP_ERR_NO_DEVICE_FOUND wenn kein KeyPilot mit dem System verbunden ist<br>
///                  ::KP_ERR_UNKNOWN wenn ein unbekannter Fehler aufgetreten ist.
DECLSPEC int WINAPI kpUSBDisconnectFromDevice(
                        int deviceID_ );
                        
/// \brief        Liefert die Systemunabh&auml;ngige Struktur des KeyPilots zur&uuml;ck
/// \param[in]    deviceID_ ID des gesuchten KeyPilots
/// \param[out]   kp_ Pointer auf die Speicherstelle, in die die gefundene Struktur abgelegt werden soll (muss allokiert sein)
/// \return       ::KP_OK bei Erfolg<br>
///               ::KP_ERR_DEVICE_NOT_FOUND wenn das angegebene Ger&auml;t nicht existiert
DECLSPEC int WINAPI kpUSBGetDevice(
                        int deviceID_,
                        KP_USB_DEV* kp_ );
                        
/// \brief        Versetzt einen KeyPilot in den Event Mode. In diesem Modus muss nicht st&auml;ndig der
///               Status des ger&auml;tes abgefragt werde, es meldet selbsst&auml;ndig &Auml;nderungen
///               beispielsweise wenn ein Schl&uuml;ssel aufgelegt oder entfernt wurde. Auch das Trennen des
///               KeyPilots von der USB Schnittstelle wird erkannt.
/// \param[in]    deviceID_  ID Des KeyPilots, dessen Event Mode aktiviert werden soll
/// \param[in]    ev_  1 um den Event Mode zu aktivieren, 0 zum deaktivieren
/// \param[in]    waitForever_  1 wenn unbegrenzt gewartet werden soll, bei 0 wird der lese Timeout auf 1 sekunde gesetzt
/// \param[in]    keyNotice_ Callback Funktion f&uuml;r empfangene Events
/// \param[in]    disconnectNotice_ Callback Funktion die aufgerufen wird, wenn die Verbindung zum KeyPilot unterbrochen wird
/// \return       ::KP_OK wenn erfolgreich<br>
///               ::KP_ERR_NO_DEVICE_FOUND wenn das angegebene Ger&auml;t nicht gefunden wurde.<br>
///               ::KP_ERR_NOT_CONNECTED wenn keine verbindung zum Ger&auml;t besteht<br>
///               ::KP_ERR_UNKNOWN wenn der Event Mode aus unbekannten Gr&uuml;nden nicht gestartet werden konnte
DECLSPEC int WINAPI kpUSBSetEventmode(
                        int deviceID_,
                        int ev_,
                        int waitForever_,
                        pfCallbackKeyNotice knCb_,
                        pfCallbackDisconnectNotice dicCb_ );
                        
/// \brief        Liest die Daten des aufgelegten Schl&uuml;ssels aus und liefrt sie in einer Struktur zur&uuml;ck.
///               Liegt kein Schl&uuml;ssel auf wird eine leere Struktur mit ung&uuml;ltigkeitsflag geliefert.
/// \param[in]    deviceID_  ID des KeyPilots, dessen Schl&uuml;ssel abgefragt werden soll
/// \param[out]   kpData_ Zeiger auf eine KP_KEYDATA Struktur die gef&uuml;llt wird (muss allokiert sein)
/// \return       ::KP_OK wenn Schl&uuml;sseldaten erfolgreich gespeichert wurden<br>
///               ::KP_ERR_NO_DEVICE_FOUND wenn das Ger&auml;t nicht existiert<br>
///               ::KP_ERR_NOT_CONNECTED wenn keine Verbindung zu dem Ger&auml;t besteht<br>
///               ::KP_ERR_UNKNOWN wenn ein unbekannter Fehler aufgetreten ist
DECLSPEC int WINAPI kpUSBGetKeyID(
                        int deviceID_,
                        KP_KEYDATA* kpData_ );
                        
/// \brief        L&auml;sst die LED in einer angegebenen Farbe fuer 1 Sekunde leuchten
/// \param[in]    deviceID_  ID des KeyPilots, dessen LED aufleuchten soll
/// \param[in]    color_  Farbe, die die LED annehmen soll. ::KP_LED_COLOR_GREEN f&uuml;r gr&uuml;n, ::KP_LED_COLOR_RED f&uuml;r rot und ::KP_LED_COLOR_OFF um die LED zu deaktivieren
/// \return       ::KP_OK wenn Befehl erfolgreich gesendet wurde<br>
///               ::KP_ERR_ILLEGAL_COLOR wenn ein ung&uuml;ltiger Farbcode angegeben wurde.<br>
///               ::KP_ERR_DEVICE_NOT_FOUND wenn das Ger&auml;t nicht gefunden wurde<br>
///               ::KP_ERR_NOT_CONNECTED wenn keine Verbindung zum Ger&auml;t besteht
DECLSPEC int WINAPI kpUSBSetLED(
                        int deviceID_,
                        int color_ );
DECLSPEC int WINAPI kpUSBDeactivateTimeout(
                        int deviceID_ );
                        
/// \brief        Schreibt Pages auf einen Schl&uuml;ssel, immer beginnend von der 1. Seite<br>
///               Die Gr&ouml;&szlig;e der Daten ergibt sich aus dem Produkt aus der Anzahl 
///               der Seiten und der Seitengr&ouml;sse.
/// \param[in]    deviceID_  ID des KeyPilots, auf den geschrieben werden soll
/// \param[in]    data_      Die Daten, die zum Ger&auml;t gesendet werden
/// \param[in]    noOfPages_ Anzahl der zu schreibenden Seiten
/// \param[in]    dataSize_  Gr&ouml;&szlig;e der Daten die geschrieben werden sollen, 
///               diese Gr&ouml;&szlig;e muss identisch der Pagegr&ouml;&szlig;e multipliziert mit der 
///               Anzahl der Seiten sein. Die Seitengr&ouml;&szlig;e kann mittels kpUSBGetKeyPageSize() ermittelt werden.
///               Die Anzahl der verf&uuml;gbaren Seiten kann mit kpUSBGetNrOfPages() erfragt werden.
/// \return       ::KP_OK bei Erfolg<br>
///               ::KP_ERR_NO_DEVICE_FOUND wenn das Ger&auml;t nicht existiert<br>
///               ::KP_ERR_NOT_CONNECTED wenn keine verbindung zum Ger&auml;t besteht<br>
///               ::KP_ERR_WRONG_KEY_TYPE wenn der falsche Schl&uuml;sseltyp aufliegt<br>
///               ::KP_ERR_WRONG_PAGE_SIZE wenn die Gr&ouml;sse der Seiten nicht stimmt<br>
///               ::KP_ERR_TO_FEW_DATA wenn zu wenige Daten vorhanden sind<br>
///               ::KP_ERR_WRONG_NR_OF_PAGES wenn die Anzahl der Seiten nicht stimmt<br>
///               ::KP_ERR_TIMEOUT wenn eine zeit&uuml;berschreitung beim schreiben auftrat<br>
///               ::KP_ERR_UNKNOWN wenn ein unbekannter Fehler auftrat
DECLSPEC int WINAPI kpUSBWritePagesToKey(
                        int deviceID_,
                        int noOfPages_,
                        char* data_,
                        int dataSize_ );
                        
/// \brief        Liest eine Seite aus einem Schl&uuml;ssel
/// \param[in]    deviceID_    ID des KeyPilots von dem gelesen werden soll
/// \param[in]    pageNo_      Die Nummer der Seite die gelesen werden soll
/// \param[out]   dataBuffer_  Zeiger auf den Datenpuffer der mit den Daten der Seite gef&uuml;llt wird
/// \param[in]    bufferSize_  Gr&ouml;&szlig;e des Datenpuffers
/// \return       ::KP_OK wenn Daten ausgelesen wurden<br>
///               ::KP_ERR_WRONG_KEY_TYPE wenn der falsche Schl&uuml;sseltyp aufliegt (z.B. einer mit zu wenig oder keinen Seiten)<br>
///               ::KP_ERR_IN_EVENTMODE wenn sich der KeyPilot im Event Modus befindet<br>
///               ::KP_ERR_NO_DEVICE_FOUND wenn das angegebene Ger&auml;t nicht stimmt<br>
///               ::KP_ERR_NOT_CONNECTED wenn keine Verbindung zum Ger&auml;t besteht
DECLSPEC int WINAPI kpUSBReadPageFromKey(
                        int deviceID_,
                        int pageNo_,
                        char* dataBuffer_,
                        int bufferSize_ );
                        
/// \brief       Liefert die Pagegr&ouml;&szlig;e des aufgelegten Schl&uuml;ssels
/// \param[in]   deviceID_ ID des KeyPilot
/// \return      Pagegr&ouml;&szlig;e des Schl&uuml;ssels.<br> 
///              0 wenn kein Schl&uuml;ssel aufliegt<br>
///              ::KP_ERR_NOT_CONNECTED wenn keine Verbindung zum Ger&auml;t besteht<br>
///              ::KP_ERR_DEVICE_NOT_FOUND wenn das Ger&auml;t nicht gefunden wurde
DECLSPEC int WINAPI kpUSBGetKeyPageSize(
                        int deviceID_ );
                        
/// \brief       Liefert die Anzahl der Verf&uuml;gbaren Pages auf dem aufgelegten Schl&uuml;ssel zur&uuml;ck
/// \param[in]   deviceID_   ID Des KeyPilots
/// \return      Anzahl der Seiten (gr&ouml;sser oder gleich 0).<br>
///              ::KP_ERR_DEVICE_NOT_FOUND wenn das Ger&auml;t nicht gefunden wurde<br>
///              ::KP_ERR_NOT_CONNECTED wenn keine Verbindung zum Ger&auml;t besteht


/// \fn           kpUSBDeactivateTimeout( int deviceID_ )
/// \brief        Deaktiviert den Timeout des KeyPilots
/// \param[in]    deviceID_  ID des KeyPilots, dessen Timeout deaktiviert werden soll
/// \return       ::KP_OK bei Erfolg<br>
///               ::KP_ERR_DEVICE_NOT_FOUND wenn das Ger&auml;t nicht gefunden wurde<br>
///               ::KP_ERR_NOT_CONNECTED wenn keine Verbindung zum Ger&auml;t besteht
DECLSPEC int WINAPI kpUSBGetNrOfPages(
                        int deviceID_ );
                        
/// \brief       liefert den zuletzt aufgetretenen Fehlercode
/// \return      der zuletzt aufgetretenen Fehlercode
DECLSPEC int WINAPI kpUSBGetLastError();

/// \brief        Liefert die Version der KeyPilotUSBDLL
/// \param[out]   version_ Buffer, in den die Versionsnummer geschrieben wird
/// \param[in]    size_  L&auml;nge des Buffers f&uuml;r die Version (Mindestgr&ouml;&szlig;e ::DLL_VERSION_LENGTH)
/// \return       ::KP_OK bei Erfolg<br>
///               ::KP_ERR_NULLPOINTER wenn der Buffer ein NULL-Pointer ist oder die angeg. Gr&ouml;&szlig;e <= 0 ist
///               ::KP_ERR_BUFFER_TOO_SMALL wenn der &uuml;bergebene Buffer zu klein ist
DECLSPEC int WINAPI kpUSBGetLibraryVersion(
                        char* version_,
                        int size_ );
                        
/// \brief        Liefert den kompletten Pfad samt Name zur KeyPilotUSBDLL
/// \param[out]   version_ Buffer, in den der Pfad geschrieben wird
/// \param[in]    size_  L&auml;nge des Buffers f&uuml;r den Name (Gr&ouml;&szlig;e ::_MAX_PATH)
/// \return       ::KP_OK bei Erfolg<br>
///               ::KP_ERR_NULLPOINTER wenn der Buffer ein NULL-Pointer ist oder die angeg. Gr&ouml;&szlig;e <= 0 ist
///               ::KP_ERR_BUFFER_TOO_SMALL wenn der &uuml;bergebene Buffer zu klein ist
DECLSPEC int WINAPI kpUSBGetLibraryPath(
                        char* path_,
                        int size_ );
                        
/// \brief        Liefert die aktuelle Firmware-Version
/// \param[in]    deviceID_    ID des KeyPilots, dessen Firmeware Version abgerufen werden soll
/// \param[out]      data_    Buffer, in dem die Versionsnummer abgelegt werden kann
/// \param[in]    size_ Gr&ouml;&szlig;e des Buffers (mindestgr&ouml;&szlig;e ::KP_FIRMWARE_VERSION_LENGTH)
/// \return          ::KP_OK bei Erfolg<br>
///               ::KP_ERR_NOT_CONNECTED wenn keine Verbindung zum Ger&auml;t besteht<br>
///               ::KP_ERR_DEVICE_NOT_FOUND wenn das Ger&auml;t nicht existiert<br>
///               ::KP_ERR_BUFFER_TOO_SMALL wenn der Buffer zu klein ist
DECLSPEC int WINAPI kpUSBGetFirmwareVersion(
                        int deviceID_,
                        char* data_,
                        int size_ );
                        
/// \brief        Versetzt einen KeyPilot in den Einrichtmodus
/// \param[in]    deviceID_  ID des KeyPilots dessen Einrichtmodus gesetzt werden soll
/// \param[in]    active_    1 aktiviert den Einrichtmodus, 0 deaktiviert ihn
/// \return       ::KP_OK bei Erfolg<br>
///               ::KP_ERR_DEVICE_NOT_FOUND wenn das Ger&auml;t nicht gefunden wurde<br>
///               ::KP_ERR_NOT_CONNECTED wenn keine Verbindung zum Ger&auml;t besteht
DECLSPEC int WINAPI kpUSBSetSettingmode(
                        int deviceID_,
                        int active_ );
                        
/// \brief       Diese Funktion liest die Seriennummer eines KeyPilots aus und liefert sie zur&uuml;ck
/// \param[in]   deviceID_  ID Des KeyPilots, von dem die Seriennummer ausgelesen werden soll
/// \param[out]  serial_    Buffer, in den die Seriennummer geschrieben werden soll
/// \param[in]   size_ Gr&ouml;&szlig;e des Buffers (Mindestgr&ouml;&szlig;e ::KP_SERIAL_NUMBER_LENGTH)
/// \return      ::KP_OK bei erfolgreichem auslesen der Seriennummer<br>
///              ::KP_ERR_DEVICE_NOT_FOUND wenn das Ger&auml;t nicht gefunden wurde<br>
///              ::KP_ERR_READ_ERROR wenn vom Ger&auml;t nicht gelesen werden konnte<br>
///              ::KP_ERR_UNKNOWN wenn ein anderer, unbekannter, Fehler aufgetreten ist.<br>
///              ::KP_ERR_BUFFER_TOO_SMALL wenn der Buffer zu klein ist
DECLSPEC int WINAPI kpUSBGetSerialNumber(
                        int deviceID_,
                        char* serial_,
                        int size_ );
                        
/// \brief           Pr&uuml;ft, ob ein Ger&auml;t mit der angegebenen DeviceID existiert
/// \param[in]       deviceID_ ID des Ger&auml;tes, desses Existenz verifiziert werden soll
/// \return          ::KP_OK wenn das Ger&auml;t existiert<br>
///                  ::KP_ERR_DEVICE_NOT_FOUND wenn das Ger&auml;t nicht existiert
DECLSPEC int WINAPI kpUSBDoesDeviceExist(
                        int deviceID_ );
                        
/// \brief       Diese Funktion liefert einen aussagekr&auml;ftigen Text String in englischer Sprache zu einer &uuml;bergebenen Fehlernummer
/// \param[in]   errCode_ Fehlernummer, deren Text String abgefragt wird.
/// \param[out]  errString_ Buffer in den der Fehlercode geschrieben werden soll
/// \param[in]   size_ Gr&ouml;&szlig;e des Buffers (Mindestgr&ouml;&szlig;e ::KP_ERRSTRING_LENGTH)
/// \return      ::KP_OK bei Erfolg<br>
///              ::KP_ERR_BUFFER_TOO_SMALL wenn der Buffer zu klein ist
DECLSPEC int WINAPI kpUSBGetErrorString(
                        int errCode_,
                        char* errString_,
                        int size_ );

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* !__KP_USBDLL_KEYPILOT_USER_DLL_H__ */
