﻿/*
 *! @brief          Hauptdialog fuer C#-Beispiel
 *
 *! @author         Markus Biggel
 * letzte Aenderung 14.07.2010
 *
 *! @addtogroup csharpusage Verwendung in C#
 *! @file Beispiele/UsageInC-Sharp/src/MainForm.cs
 *! @ingroup csharpusage csharp csharpusageev
 */
using System;
using System.Threading;
using System.Windows.Forms;
using KeyPilotCom;


namespace keypilot
{
    /// <summary>
    /// Description of MainForm.
    /// </summary>
    public partial class MainForm : Form
    {
        kpDialog                        m_kpDlg;

        public delegate void KpEventGUIDelegate(
                                 int deviceID_,
                                 KP_KEYDATA kpData_,
                                 bool disconnected_ );


        public MainForm()
        {
            //
            // The InitializeComponent() call is required for Windows Forms designer support.
            //
            InitializeComponent();
            m_kpDlg = new kpDialog();

            // DLL-Informationen ausfuellen:
            String dummy = "";
            KP_RETURN result;
            result = KeyPilot.kpUSBGetLibraryVersion( out dummy );
            if ( result == KP_RETURN.KP_OK ) {
                dllVersion.Text = dummy;
            } else {
                dllVersion.Text = "-";
            }

            result = KeyPilot.kpUSBGetLibraryPath( out dummy );
            if ( result == KP_RETURN.KP_OK ) {
                dllPath.Text = dummy;
            } else {
                dllPath.Text = "-";
            }
        }


        void ExitToolStripMenuItemClick(
                                    object sender_,
                                    EventArgs evArgs_ )
        {
            Application.Exit();
        }


        void KeyPilotToolStripMenuItemClick(
                                    object sender_,
                                    EventArgs evArgs_ )
        {
            m_kpDlg.ShowDialog();
            if ( m_kpDlg.DeviceID >= 0 ) {
                btnReadKey.Enabled = true;
                btnReadPages.Enabled = true;
                btnEventMode.Enabled = true;
                statusBarText.Text = "KeyPilot verbunden";
                KP_USB_DEV kpDevice = new KP_USB_DEV();
                KeyPilot.kpUSBGetDevice( m_kpDlg.DeviceID, ref kpDevice );
                kpSerialNo.Text = kpDevice.serialNumber;

                String version = "";
                KeyPilot.kpUSBGetFirmwareVersion( m_kpDlg.DeviceID, out version );
                kpFirmware.Text = version;

                kpName.Text = kpDevice.prName;;
                kpDescription.Text = kpDevice.description;
            } else {
                btnReadKey.Enabled = false;
                btnReadPages.Enabled = false;
                btnEventMode.Enabled = false;
                statusBarText.Text = "Kein KeyPilot verbunden";
                kpSerialNo.Text = "-";
                kpFirmware.Text = "-";
                kpName.Text = "-";
                kpDescription.Text = "-";
            }
        }


        void btnReadKey_Click(      object sender_,
                                    EventArgs evArgs_ )
        {
            if ( m_kpDlg.DeviceID < 0 ) {
                return;
            }

            KP_KEYDATA kpData = new KP_KEYDATA();
            KP_RETURN result;
            result = KeyPilot.kpUSBGetKeyID( m_kpDlg.DeviceID, ref kpData );
            if ( result == KP_RETURN.KP_OK ) {
                if ( kpData.validData ) {
                    keyID.Text = kpData.keyID;
                    famCode.Text = kpData.famCode;

                    //btnReadPages.Enabled = true;
                } else {
                    keyID.Text = "Kein Schlüssel aufgelegt";
                    famCode.Text = "-";

                    //btnReadPages.Enabled = false;
                }
            }
        }


        void btnReadPages_Click(    object sender_,
                                    EventArgs evArgs_ )
        {
            if ( m_kpDlg.DeviceID < 0 ) {
                return;
            }

            Cursor.Current = Cursors.WaitCursor;

            int numPages;
            int pageSize;
            byte[] page;

            // erstmal einen aufliegenden Schluessel finden:
            btnReadKey_Click( this, null );
            if ( m_kpDlg.DeviceID < 0 ) {
                return;
            }

            numPages = KeyPilot.kpUSBGetNrOfPages( m_kpDlg.DeviceID );
            if ( numPages <= 0 ) {
                pagesContent.Text = "Keine Inhaltsdaten vorhanden";
                Cursor.Current = Cursors.Default;
                return;
            }
            pageSize = KeyPilot.kpUSBGetKeyPageSize( m_kpDlg.DeviceID );
            if ( pageSize <= 0 ) {
                pagesContent.Text = "Seitengrösse kann nicht bestimmt werden";
                Cursor.Current = Cursors.Default;
                return;
            }
            page = new byte[ pageSize ];

            statusBarText.Text = "Lese " + numPages.ToString() + " Datenseiten ein...";

            pagesContent.Text = "";
            for ( int i = 0; i <= numPages; i++ ) {
                KeyPilot.kpUSBReadPageFromKey( m_kpDlg.DeviceID,
                                               i, 
                                               page, 
                                               pageSize );
                if ( i > 0 ) {
                    pagesContent.AppendText( Environment.NewLine );
                }
                pagesContent.AppendText( i.ToString( "d2" ) + ": " );

                pagesContent.AppendText( byte2hexstring(
                                           page,
                                           4, " ",
                                           16, Environment.NewLine + "    ") );
            }

            statusBarText.Text = "Inhaltsdaten eingelesen.";

            Cursor.Current = Cursors.Default;
        }


        string byte2hexstring(      byte[] data_,
                                    int group1Size_,
                                    string group1Seperator_,
                                    int group2Size_,
                                    string group2Seperator_ )
        {
            String str = "";
            for ( int i = 0; i < data_.Length; i++ ) {
                if ( group2Size_ > 0 && i > 0 && (( i % group2Size_ ) == 0) ) {
                    str += group2Seperator_;
                } else
                if ( group1Size_ > 0 && i > 0 && (( i % group1Size_ ) == 0) ) {
                    str += group1Seperator_;
                }
                str += string.Format( "{0:X2} ", ((int)data_[ i ]) );
            }
            return str;
        }


        private void btnEventMode_Click(
                                    object sender_,
                                    EventArgs evArgs_ )
        {
            lock( this ) {
                Cursor.Current = Cursors.WaitCursor;

                KPCallBack kpCB = KPCallBack.getInstance();

                if ( !kpCB.EventModeStarted ) {
                    // Eventmodus ist nicht aktiv
                    // --> aktivieren

                    // die Buttons kurz "wegschalten"
                    btnEventMode.Enabled = false;
                    btnReadKey.Enabled = false;

                    // mal kurz die LED auf "rot" setzen (einfach so ;-):
                    KeyPilot.kpUSBSetLED( m_kpDlg.DeviceID,
                                          KP_LED_COLORS.KP_LED_COLOR_RED );

                    btnEventMode.Text = "&Eventmodus aus";

                    // schnell noch Einlesen, ob aktuell ein Schluessel aufliegt:
                    btnReadKey_Click( this, null );

                    kpCB.startEventMode( m_kpDlg.DeviceID, kpEvent );

                    // Buttons wieder freischalten:
                    btnEventMode.Enabled = true;
                    btnReadKey.Enabled = true;
                } else {
                    // Eventmodus ist aktiv
                    // --> deaktivieren

                    // die Buttons kurz "wegschalten"
                    btnEventMode.Enabled = false;
                    btnReadKey.Enabled = false;

                    kpCB.stopEventMode( m_kpDlg.DeviceID, kpEvent );

                    // schnell noch Einlesen, ob aktuell ein Schluessel aufliegt:
                    btnReadKey_Click( this, null );

                    btnEventMode.Text = "&Eventmodus ein";

                    // Buttons wieder freischalten:
                    btnEventMode.Enabled = true;
                    btnReadKey.Enabled = true;
                }

                Cursor.Current = Cursors.Default;
            }
        }


        public void kpEvent(        int deviceID_,
                                    KP_KEYDATA kpData_,
                                    bool disconnected_ )
        {
            if ( deviceID_ < 0 || deviceID_ != m_kpDlg.DeviceID ) {
                // interessiert hier nicht...
                return;
            }

            // zuerst alle nicht GUI relevanten Arbeiten erledigen:
            if ( disconnected_ ) {
                // KeyPilot wurde abgesteckt
                // --> Eventmodus stoppen
                KPCallBack kpCB = KPCallBack.getInstance();

                kpCB.stopEventMode( deviceID_, kpEvent );
            }

            // dann GUI relevante Arbeiten:
            if ( InvokeRequired ) {
                // fremder Thread;
                //   --> indirekter Aufruf (Weiterleitung an GUI-Thread)
                Invoke( new KpEventGUIDelegate(kpEventGUIUpdate),
                        deviceID_,
                        kpData_,
                        disconnected_ );
                Thread.Sleep( 0 );
            } else {
                // wir sind im GUI-Thread
                //   --> direkter Aufruf moeglich
                kpEventGUIUpdate( deviceID_, kpData_, disconnected_ );
            }
        }


        protected void kpEventGUIUpdate(
                                    int deviceID_,
                                    KP_KEYDATA kpData_,
                                    bool disconnected_ )
        {
            lock( this ) {
                if ( disconnected_ ) {
                    // KeyPilot wurde abgesteckt
                    // --> Buttons disablen:
                    btnReadKey.Enabled = false;
                    btnReadPages.Enabled = false;
                    btnEventMode.Enabled = false;
                    btnEventMode.Text = "&Eventmodus ein";
                } else
                if ( kpData_.command.Equals("KEY_PRESENT",
                            StringComparison.InvariantCultureIgnoreCase)
                       && kpData_.validData ) {
                    keyID.Text = kpData_.keyID;
                    famCode.Text = kpData_.famCode;

                    btnReadKey.Enabled = true;
                    btnReadPages.Enabled = true;
                } else
                if ( kpData_.command.Equals("KEY_REMOVED",
                            StringComparison.InvariantCultureIgnoreCase) ) {
                    keyID.Text = "Kein Schlüssel aufliegend";
                    famCode.Text = "-";

                    btnReadKey.Enabled = false;
                    btnReadPages.Enabled = false;
                }
            }
        }
    }
}