#include <stdio.h>
#include "KeyPilotUSBUserDLL.h"

/*! \addtogroup cevent Eventmode in C
    \file EventMode/main.c
     \ingroup cevent
*/

int deviceID = -1; /*!< ID des Ger&auml;tes*/

int main(int argc, char** argv);
void WINAPI keyPilotEventCallback(int _deviceID,PKEYPILOT_KEYDATA keyData);
void WINAPI keyPilotDisconnectCallback(int _deviceID);

/*! \fn main(int argc, char** argv)
    \ingroup cevent
    \brief Main Funktion, welche einen freien KeyPilot sucht und auf eingehende Events wartet
     \param[in] argc Anzahl der Kommandozeilenparameter
     \param[in] argv Liste der Kommandozeilenparameter
 \code 
 /**/
int main(int argc, char** argv)
{
    int i;
    int kpCount;
    KEYPILOT_USB_DEVICE* kpList = NULL;
    PKEYPILOT_KEYDATA kpData = (PKEYPILOT_KEYDATA)malloc(sizeof(PKEYPILOT_KEYDATA));
    char errStr[KP_ERRSTRING_LENGTH];

    // Scanning for KeyPilots
    kpUSBScanDevices();
    kpCount = kpUSBGetNumberOfDevices();
    if(kpCount <= 0)
    {
        printf("No KeyPilots connected to the system\n");
        return 0;
    }
    printf("Number of KeyPilots found: %d\n",kpCount);

    // find the first available KeyPilot
    if(kpList != NULL)
        free(kpList);
    kpList = malloc(sizeof(KEYPILOT_USB_DEVICE)*kpCount);
    kpUSBGetDeviceList(kpList,kpCount);
    for(i=0;i<kpCount;i++)
    {
        if(kpList[i].connectionStatus == AVAILABLE)
        {
            deviceID = kpList[i].deviceID;
            break;
        }
    }
    if(deviceID < 0)
    {
        printf("No KeyPilot available\n");
        return 0;
    }

    // connect to the KeyPilot, read the Key and disconnect
    if(kpUSBConnectToDevice(deviceID) != KP_OK)
    {
        kpUSBGetErrorString(kpUSBGetLastError(),errStr,KP_ERRSTRING_LENGTH);
        printf("Error: %s\n",errStr);
        return 0;
    }

    // Activating the Event Mode
    kpUSBSetEventmode(deviceID,1,1,keyPilotEventCallback,keyPilotDisconnectCallback);
    while(deviceID >= 0) {
        Sleep( 50 );
    }
    return 0;
}
/*! \endcode */


/*! \fn keyPilotEventCallback(int _deviceID,PKEYPILOT_KEYDATA keyData)
    \ingroup cevent
    \brief Callback Funktion f&uuml;r eingehende Events
    \param[in]  _deviceID ID des KeyPilots, f&uuml;r den ein Event ankam
     \param[in]  keyData Daten die ankamen
\code /**/
void WINAPI keyPilotEventCallback(int _deviceID,PKEYPILOT_KEYDATA keyData)
{
    printf("--------------------\n");
    printf("Data Received:\n");
    if(keyData->validData)
    {
        printf("Key ID: %s\n",keyData->keyID);
        printf("Key Family Code: %s\n",keyData->famCode);
    }
    else
        printf("Key removed\n");
}
/*! \endcode */

/*! \fn keyPilotDisconnectCallback(int _deviceID)
    \ingroup cevent
    \brief Callback Methode, falls der KeyPilot getrennt wurde
    \param[in]  _deviceID ID des KeyPilots, der getrennt wurde
     \code
/**/
void WINAPI keyPilotDisconnectCallback(int _deviceID)
{
    printf("Disconnect from device\n");
    deviceID = -1;
}
/*! \endcode  */
