#ifndef KEYPILOT_CALLBACK_CLASS_HEADER
#define KEYPILOT_CALLBACK_CLASS_HEADER

/*!  \namespace KeyPilotClass
     \file KeyPilotCallbackClass.h
     \ingroup cpp
*/


#include <vector>
#include "KeyPilotUSBUserDLL.h"
#include "KeyPilotEv.h"

using namespace std;

/*! \struct RegisteredKPEventClasses 
    \ingroup cpp
    \brief Struktur mit dem Event Objekt f&uuml;r die Callback Methoden und der device ID des Ger&auml;tes
*/
struct RegisteredKPEventClasses
{
    int deviceID;         /*!<ID des KeyPilots  */
    KeyPilotEv* object;   /*!<Objekt f&uuml;r den Callback*/
};

/*! \ingroup cpp
    \brief Klasse f&uuml;r den Callback des Event Modus
*/
class KeyPilotCallbackClass
{
protected:
    /*! Constructor */
    KeyPilotCallbackClass();
    /*! Destructor */
    ~KeyPilotCallbackClass();
    /*!&Uuml;bermittelt die Empfangenen Daten an die Zielklasse 
       \param[in] deviceID    ID des KeyPilots
       \param[in] keyData     Datenstruktur mit KeyDaten
       \param[in] disconnect  wenn true wurde der KeyPilot getrennt, sonst sind g&uuml;ltige Daten vorhanden
    */
    void throwEvent(int deviceID, PKEYPILOT_KEYDATA keyData, bool disconnect=false); 
    vector<RegisteredKPEventClasses*>  m_objectList; /*!< Liste der registrierten Callback Klassen f&uuml;r den Event Modus*/
public:
    /*! Methode um die Singleton Instanz zu holen  
        \return  Instanz auf die Singleton Klasse
    */
    static KeyPilotCallbackClass* getInstance();  

    /*! Statische Callback Funktion, die von der KeyPilot DLL aufgerufen wird wenn ein Event am KeyPilot auftritt 
        \param[in]     deviceID ID des KeyPilots, f&uuml;r den Daten angekommen sind
         \param[in]     keyData Daten, die vom KeyPilot angekommen sind
    */
    static void WINAPI keyPilotNotice(int deviceID,PKEYPILOT_KEYDATA keyData);

    /*! Statische Callback Methode die aufgerufen wird, soblad der KeyPilot getrennt wird 
        \param[in]   deviceID ID des KeyPilots, dessen Verbindung gerennt werden soll
    */
    static void WINAPI keyPilotDisconnect(int deviceID); 

    /*! Methode um eine Callback Klasse f&uuml;r einen KeyPilot festzulegen (Eventmodus wird aktiviert)
        \param[in]    deviceID ID des KeyPilots, f&uuml;r den eine Callback Klasse registriert werden soll
         \param[in]    ev Callback Klasse
    */
    void registerClass(int deviceID, KeyPilotEv* ev); 

    /*! Methode um die Callback Klasse wieder von einem KeyPilot zu trennen (Eventmodus wird deaktiviert) 
        \param[in]   deviceID ID des KeyPilots, dessen Eventmodus deaktiviert werden soll
        \param[in]   ev Callback Klasse des KeyPilots, dessen Event Modus deaktiviert werden soll
    */
    void deregisterClass(int deviceID, KeyPilotEv* ev); 
};

#endif
