#include "KeyPilotClass.h"

/*! \file KeyPilotClass.cpp
    \ingroup cpp
*/

KeyPilotClass::KeyPilotClass()
{
    m_deviceID = -1;
    m_connected = false;
}

KeyPilotClass::KeyPilotClass(int deviceID)
{
    m_deviceID = deviceID;
}

KeyPilotClass::~KeyPilotClass()
{
    disconnetFromDevice();
}

bool KeyPilotClass::connectToDevice()
{
    if(m_connected)
        return true;
    if( m_deviceID >= 0 && kpUSBConnectToDevice(m_deviceID) == KP_OK )
        m_connected = true;
    else
        m_connected = false;
    return m_connected;
}

void KeyPilotClass::disconnetFromDevice()
{
    if(m_deviceID >= 0)
        kpUSBDisconnectFromDevice(m_deviceID);
    m_connected = false;
}

PKEYPILOT_KEYDATA KeyPilotClass::getKeyData()
{
    if(!m_connected)
        return NULL;
    KEYPILOT_KEYDATA *kpData = new KEYPILOT_KEYDATA;
    kpUSBGetKeyID(m_deviceID,kpData);
    return kpData;
}

vector<KEYPILOT_USB_DEVICE> KeyPilotClass::getKeyPilotList()
{
    vector<KEYPILOT_USB_DEVICE> deviceList;
    int numOfDevs;

    kpUSBScanDevices();
    numOfDevs = kpUSBGetNumberOfDevices();
    KEYPILOT_USB_DEVICE *devList = new KEYPILOT_USB_DEVICE[numOfDevs];
    kpUSBGetDeviceList(devList,numOfDevs);
    deviceList.clear();
    for(int i=0;i<numOfDevs;i++)
    {
        deviceList.push_back(devList[i]);
    }

    return deviceList;
}

bool KeyPilotClass::isAvailable()
{
    kpUSBScanDevices();
    int numOfDevs = kpUSBGetNumberOfDevices();
    KEYPILOT_USB_DEVICE *devList = new KEYPILOT_USB_DEVICE[numOfDevs];
    kpUSBGetDeviceList(devList,numOfDevs);
    for(int i=0;i<numOfDevs;i++)
    {
        if(devList[i].deviceID == m_deviceID)
        {
            if(devList[i].connectionStatus == AVAILABLE)
                return true;
            else
                return false;
        }
    }
    return false;
}

bool KeyPilotClass::isConnected()
{
    return m_connected;
}

void KeyPilotClass::setDeviceID(int deviceID)
{ 
    m_deviceID = deviceID; 
}

void KeyPilotClass::keyPilotEvent(PKEYPILOT_KEYDATA keyData, bool disconnect)
{
    printf("\n\n------------------------------\n");
    printf("got event: ");
    if(disconnect)
    {
        printf("KeyPilot disconnected\n");
        disconnetFromDevice();
        m_connected = false;
        return;
    }
    if(keyData != NULL)
    {
        printf("Data from KeyPilot\n");
        if(keyData->validData != 0)
        {
            printf("Key ID: %s\n",keyData->keyID);
            printf("Family Code: %s\n",keyData->famCode);
        }
        else
            printf("Key Removed");
    }
}
