#ifndef KEYPILOT_CLASS_HEADER
#define KEYPILOT_CLASS_HEADER

/*!  \file KeyPilotClass.h
     \ingroup cpp
*/

#include <vector>
#include "KeyPilotEv.h"
#include "KeyPilotCallbackClass.h"

using namespace std;

/*! \brief Klasse zur behandlung des KeyPilots. Abgeleitet von KeyPilotEv
    \ingroup cpp
*/
class KeyPilotClass : public KeyPilotEv
{
public:
    /*! Statische Funktion, welche eine Liste aller verf&uuml;gbarer KeyPilots ausgibt 
        \return   Liste der KeyPilots
    */
    static vector<KEYPILOT_USB_DEVICE> getKeyPilotList(); 
public:
    KeyPilotClass(); /*!<Constructor*/
    /*! &Uuml;berladener Constructor, dem man gleich die zust&auml;ndige DeviceID des KeyPilots mit &uuml;bergibt
        \param[in]  deviceID ID des KeyPilots, f&uuml;r den das Objekt zust&auml;ndig ist
    */
    KeyPilotClass(int deviceID); 
    ~KeyPilotClass(); /*!<Destructor */
    /*! Datenstruktur um die letzten empfangenen Daten abholen
        \return Sckl&uuml;sseldaten des zuletzt ausgelesenen Schl&uuml;ssels 
    */
    PKEYPILOT_KEYDATA getKeyData();
    /*! baut eine Verbindung zu einem KeyPilot auf
        \return Erfolgsmeldung
    */
    bool connectToDevice();
    /*! Setzt die DeviceID des KeyPilots, der Verwendet werden soll
        \param[in]   deviceID ID des KeyPilots
    */
    void setDeviceID(int deviceID);
    void disconnetFromDevice();   /*!< trennt die verbindung von einem KeyPilot, falls diese besteht*/
    /*! Pr&uuml;ft, ob eine Verbindung zu einem KeyPilot besteht
        \return true wenn eine Verbindung besteht, sonst false
    */
    bool isConnected();
    /*! Pr&uuml;ft, ob der KeyPilot mit der gesetzten ID verf&uuml;gbar ist
        \return true wenn verf&uuml;gbar, sonst false
    */
    bool isAvailable();
    /*! Nimmt die Daten entgegen, welche vom KeyPilot empfangen wurden
        \param[in] keyData Daten des Schl&uuml;ssels
        \param[in] disconnect wenn true wurde die Verbindung vom KeyPilot getrennt, ansonsten sind die Schl&uuml;sseldaten g&uuml;ltig
    */
    void keyPilotEvent(PKEYPILOT_KEYDATA keyData, bool disconnect);
protected:
    bool m_connected; /*!<bei true besteht eine offene Verbindung*/
};

#endif
