#ifndef KEYPILOT_EVENT_HEADER
#define KEYPILOT_EVENT_HEADER

/*! \file KeyPilotEv.h
    \ingroup cpp
*/

#include "KeyPilotUSBUserDLL.h"

/*! \ingroup cpp
    \brief Abstarkte Klasse um Ergebnisse der Events vom Eventmode entgegen zu nehmen
*/
class KeyPilotEv
{
public:
    KeyPilotEv(); /*!<Constructor*/
    /*! &Uuml;berladener Constructor, welcher gleich die DeviceID &uuml;bergeben bekommt
        \param[in]  deviceID ID des KeyPilot
    */
    KeyPilotEv(int deviceID);
    ~KeyPilotEv(); /*!<Destructor*/
    /*! Startet oder beendet den Eventmode
        \param[in]   evm bei true wird der Event Mode aktiviert, bei false deaktiviert
    */
    void setEventmode(bool evm);
    /*! pr&uuml;ft, ob der Eventmode aktiv ist
        \return true = Eventmodus ist gesetzt, false = Eventmode nicht gesetzt
    */
    bool isInEventMode(); 
    /*! Setzt eine DeviceID wenn keine gesetzt ist. Setzen ist nur m&ouml;glich, wenn sich der KeyPilot nicht im Eventmode befinder
        \param[in]  id DeviceID die gesetzt werden soll
    */
    virtual void setDeviceID(int id);
    /*! Pure Virtual Methode, diese Methode verarbeitet die eingehenden Daten.
        \param[in] keyData     Daten, die vom KeyPilot ankommen
         \param[in] disconnect  bei true wurde die Verbindung getrennt
    */
    virtual void keyPilotEvent(PKEYPILOT_KEYDATA keyData, bool disconnect) = 0;
protected:
    int m_deviceID; /*!<ID des Ger&auml;tes, f&uuml;r das dieses Objekt zust&auml;ndig ist*/
    bool m_eventMode; /*!<bei true befindet sich der KeyPilot im Eventmode, bei false nicht*/
};


#endif
