import java.util.Scanner;
public class KeyScanner {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
		Scanner in = new Scanner(System.in);	
		Scanner data = new Scanner(System.in);
		String command = "";
		String dataToWrite;
		
		int ErrorCode;											//Variable holding the functions return value
		int nrOfDevices;										//Number of KP devices found
		int deviceID;											//Device ID
		int pageNr;												//Page number to read data from
		int pageSize;											//Page size
		int pageAmount;											//Amount of pages to read / write
		KEYPILOT_USB_DEVICE kp = new KEYPILOT_USB_DEVICE();		//Auxiliary KP Class variable for printing error messages
		KEYPILOT_USB_DEVICE[] kpArray;							//Array holding the present KP Devices. Is initialized later
		KEYPILOT_KEYDATA keyData = new KEYPILOT_KEYDATA();		//Variable holding the Key information
		byte[] dataBuffer;										//Variable used to read or write to a Key
		
		/*
		 * The Wrapper DLL has to be loaded into the program
		 * Be sure to keep the DLL in the same directory as the application
		 */
		System.loadLibrary("KeyPilotUSBJavaWrapper");
		
		Line();
		Line("KP USB Java Test Program");
		
		while(!command.equals("q")){
			Line();
			Line("Commands: ");
			Line("s - Scan devices");
			Line("l - List devices");
			Line("c - Connect to device");
			Line("d - Disconnect from device");
			Line("k - Get key data");
			Line("r - Read page from key");
			Line("w - Write data to key");
			Line("e - Get last error message");
			Line("q - Quit program");
			
			command = in.next();
			
			/*
			 * Command decoding.
			 * Usually the user has to scan the USB devices on the PC
			 */
			if(command.equals("s")){
				Line("#################################################");
				
				/*
				 * In most of the examples the error code is evaluated
				 */
				ErrorCode = KEYPILOT_USB_DEVICE.kpUSBScanDevices();
				if(ErrorCode != KEYPILOT_USB_DEVICE.KP_OK){
					KEYPILOT_USB_DEVICE.kpUSBGetErrorString(ErrorCode, kp);
					Line("Error #" + ErrorCode + ": " + kp.getErrorString());
					Line();
				}
				else{
					Line("Scan devices Ok...");
					Line("#################################################");
				}
			}
			
			/*
			 * This command lists the connected devices to the PC
			 * First it gets the number of devices connected and then
			 * fills an array with the devices found. Finally it prints out
			 * the devices characteristics.
			 */
			if(command.equals("l")){
				Line("#################################################");
				nrOfDevices = KEYPILOT_USB_DEVICE.kpUSBGetNumberOfDevices();
				
				//No devices present
				if(nrOfDevices == KEYPILOT_USB_DEVICE.KP_OK){
					Line("No devices found");
					Line();
				}
				//At least one device present since the last scan
				else if (nrOfDevices > KEYPILOT_USB_DEVICE.KP_OK){
					
					//Initializing the amount of elements in an array
					kpArray = new KEYPILOT_USB_DEVICE[nrOfDevices];
					for(int i = 0; i < nrOfDevices; i++){
						
						//Each element of the array has to be initialized properly,
						//otherwise the Java Native Interface throws an exception
						kpArray[i] = new KEYPILOT_USB_DEVICE();
						
						ErrorCode = KEYPILOT_USB_DEVICE.kpUSBGetDeviceListIndex(kpArray[i], i);
						if(ErrorCode != KEYPILOT_USB_DEVICE.KP_OK){
							KEYPILOT_USB_DEVICE.kpUSBGetErrorString(ErrorCode, kp);
							Line("Error #" + ErrorCode + ": " + kp.getErrorString());
							Line();
						}
						else{
							Line();
							printKpUSBDev(kpArray[i]);
						}
					}
					Line();
					Line("List Devices Ok");
				}
				//Error executing the command
				else{
					KEYPILOT_USB_DEVICE.kpUSBGetErrorString(nrOfDevices, kp);
					Line("Error #" + nrOfDevices + ": " + kp.getErrorString());
					Line();
				}
				Line("#################################################");
			}
			
			/*
			 * Here a connection is established to a KP using the deviceID.
			 * After succesful connection the connection status is shown
			 */
			if(command.equals("c")){
				Line("#################################################");
				System.out.print("Select device: ");
				deviceID = in.nextInt();
				Line("Connecting to device " + deviceID + "...");
				
				ErrorCode = KEYPILOT_USB_DEVICE.kpUSBConnectToDevice(deviceID);
				if(ErrorCode != KEYPILOT_USB_DEVICE.KP_OK){
					KEYPILOT_USB_DEVICE.kpUSBGetErrorString(ErrorCode, kp);
					Line("Error #" + ErrorCode + ": " + kp.getErrorString());
				}
				else{
					Line();
					//The actual connection status is updated to the kp variable
					KEYPILOT_USB_DEVICE.kpUSBGetDevice(deviceID, kp);
					Line("Connection Status: " + kp.getConnectionStatus());										
				}
				Line("#################################################");
			}
			
			/*
			 * Here we try to disconnect from a device to which a connection
			 * has been previously established
			 */
			if(command.equals("d")){
				Line("#################################################");
				Line();
				System.out.print("Select device: ");
				deviceID = in.nextInt();
				Line("Disconnecting from device " + deviceID + "...");
				
				ErrorCode = KEYPILOT_USB_DEVICE.kpUSBDisconnectFromDevice(deviceID);
				if(ErrorCode != KEYPILOT_USB_DEVICE.KP_OK){
					KEYPILOT_USB_DEVICE.kpUSBGetErrorString(ErrorCode, kp);
					Line("Error #" + ErrorCode + ": " + kp.getErrorString());
				}
				else{
					Line();
					//The actual connection status is updated to the kp variable
					KEYPILOT_USB_DEVICE.kpUSBGetDevice(deviceID, kp);
					Line("Connection Status: " + kp.getConnectionStatus());										
				}
				Line("#################################################");
			}
			
			/*
			 * Here the information from a Key hanging on a KP is read
			 */
			if(command.equals("k")){
				Line("#################################################");
				Line();
				System.out.print("Select device: ");
				deviceID = in.nextInt();
				Line("Getting key data from device " + deviceID + "...");
				
				ErrorCode = KEYPILOT_USB_DEVICE.kpUSBGetKeyID(deviceID, keyData);
				if(ErrorCode != KEYPILOT_USB_DEVICE.KP_OK){
					KEYPILOT_USB_DEVICE.kpUSBGetErrorString(ErrorCode, kp);
					Line("Error #" + ErrorCode + ": " + kp.getErrorString());
				}
				else{
					Line();
					printKeyData(keyData);										
				}
				Line("#################################################");
			}
			
			/*
			 * This command reads information from a Key. 
			 */
			if(command.equals("r")){
				Line("#################################################");
				Line();
				System.out.print("Select device: ");
				deviceID = in.nextInt();
				System.out.print("Select page (First page is 0): ");
				pageNr = in.nextInt();
				System.out.print("Amount of pages: ");
				pageAmount = in.nextInt();
				Line("Reading page " + pageNr + "...");
				
				//The size of the key is determined
				pageSize = KEYPILOT_USB_DEVICE.kpUSBGetKeyPageSize(deviceID);
				if(pageSize < KEYPILOT_USB_DEVICE.KP_OK){
					KEYPILOT_USB_DEVICE.kpUSBGetErrorString(pageSize, kp);
					Line("Error #" + pageSize + ": " + kp.getErrorString());
				}
				//If the page size is too small, no data is read
				else if(pageSize == KEYPILOT_USB_DEVICE.KP_OK){
					Line("Not able to read, page size is 0 bytes");
				}
				else{
					//The databuffer receives the information of the size page
					dataBuffer = new byte[pageSize];
					Line("Page Information (Hex): ");
					for(int i = 0; i < pageAmount; i++){
						
						//The read page is stored in the data buffer
						ErrorCode = KEYPILOT_USB_DEVICE.kpUSBReadPageFromKey(deviceID, pageNr, dataBuffer, pageSize);
						if(ErrorCode != KEYPILOT_USB_DEVICE.KP_OK){
							KEYPILOT_USB_DEVICE.kpUSBGetErrorString(ErrorCode, kp);
							Line("Error #" + ErrorCode + ": " + kp.getErrorString());
						}
						else{
							//The data buffer is printed out on screen
							for(int j = 0; j < dataBuffer.length; j++){
								System.out.print(Integer.toHexString(dataBuffer[j]) + " ");
							}
						}
					pageNr++;
					}
					Line();
					Line("End Information (Hex)");
				}
				Line("#################################################");
			}
			
			/*
			 * This command writes typed information on a key. Depending on how
			 * much was typed, the data is filled. If the data does not fill a
			 * complete page, the rest is filled with zeros
			 */
			if(command.equals("w")){
				Line("#################################################");
				Line();
				System.out.print("Select device: ");
				deviceID = in.nextInt();
				Line("Write data (Ascii): ");
				dataToWrite = "";
				dataToWrite = data.nextLine();
				Line("Writing page...");
				dataBuffer = dataToWrite.getBytes();
				
				//Determine the page size
				pageSize = KEYPILOT_USB_DEVICE.kpUSBGetKeyPageSize(deviceID);
				if(pageSize < KEYPILOT_USB_DEVICE.KP_OK){
					KEYPILOT_USB_DEVICE.kpUSBGetErrorString(pageSize, kp);
					Line("Error #" + pageSize + ": " + kp.getErrorString());
					Line();
					in.reset();
				}
				//If the page size is too small, no data is written
				else if(pageSize == KEYPILOT_USB_DEVICE.KP_OK){
					Line("Not able to read, page size is 0 bytes");
					Line();
					in.reset();
				}
				else{
					//Determine the amount of pages to write
					pageAmount = dataBuffer.length / pageSize;
					if((dataBuffer.length % pageSize) != 0 ) pageAmount++;
					
					ErrorCode = KEYPILOT_USB_DEVICE.kpUSBWritePagesToKey(deviceID, pageAmount, dataBuffer, pageAmount * pageSize);
					if(ErrorCode != KEYPILOT_USB_DEVICE.KP_OK){
						KEYPILOT_USB_DEVICE.kpUSBGetErrorString(ErrorCode, kp);
						Line("Error #" + ErrorCode + ": " + kp.getErrorString());
						Line();
					}
					else{
						Line("Write successful");
						Line();
					}
					in.reset();
					Line();
				}
				Line("#################################################");
			}
			
			/*
			 * This command gets the last error message produced by the selected
			 * device
			 */
			if(command.equals("e")){
				Line("#################################################");
				Line();
				System.out.print("Select device: ");
				deviceID = in.nextInt();
				
				ErrorCode = KEYPILOT_USB_DEVICE.kpUSBGetLastError();
				KEYPILOT_USB_DEVICE.kpUSBGetErrorString(ErrorCode, kp);
				Line("Last error: ");
				Line("Error #" + ErrorCode + ": " + kp.getErrorString());
				Line();
				Line("#################################################");
			}
		}		
	}
	
	/*
	 * This is a helper function to print on screen the device information
	 */
	private static void printKpUSBDev(KEYPILOT_USB_DEVICE kp){
		System.out.println("Serial Number: " + kp.getSerialNumber());
		System.out.println("Vendor Name: " + kp.getVendorName());
		System.out.println("Friendly Name: " + kp.getFriendlyName());
		System.out.println("Description: " + kp.getDescription());
		System.out.println("Product Name: " + kp.getPrName());
		System.out.println("Vendor ID: 0x" + Integer.toHexString(kp.getVendorID()));
		System.out.println("Product ID: 0x" + Integer.toHexString(kp.getProductID()));
		System.out.println("Version: 0x" + Integer.toHexString(kp.getVersion()));
		System.out.println("Device ID: " + kp.getDeviceID());
		System.out.println("Connection Status: " + kp.getConnectionStatus());
		System.out.println("Event Mode: " + kp.getEventMode());		
	}
	
	/*
	 * This is a helper function to print out the Key information
	 */
	private static void printKeyData(KEYPILOT_KEYDATA kd){
		System.out.println("Valid Data: " + kd.getValidData());
		System.out.println("Command: " + kd.getCommand());
		System.out.println("Key ID: " + kd.getKeyID());
		System.out.println("Family Code: " + kd.getFamCode());
		System.out.println("CRC: " + kd.getCRC());	
	}
	
	/*
	 * Helper functions to print lines
	 */
	private static void Line(String s){
		System.out.println(s);
	}
	
	private static void Line(){
		System.out.println();
	}
}