import ctypes
from ctypes import *
import struct
import string
import sys
import time

# Return Codes
KP_OK                    = 0
KP_ERR_NO_DEVICE_FOUND   = -1
KP_ERR_CONNECTION_FAILED = -4
KP_ERR_NOT_CONNECTED     = -5
KP_ERR_WRONG_PAGE_SIZE   = -6
KP_ERR_BUFFER_TOO_SMALL  = -7
KP_ERR_DEVICE_BUSY       = -10
KP_ERR_DEVICE_NOT_FOUND  = -11
KP_ERR_UNKNOWN           = -12
KP_ERR_ILLEGAL_DEVICE    = -13
KP_ERR_DATASIZE_WRONG    = -14
KP_ERR_READ_ERROR        = -15
KP_ERR_TIMEOUT           = -16
KP_ERR_WRONG_NR_OF_PAGES = -17
KP_ERR_WRONG_KEY_TYPE    = -18
KP_ERR_IN_EVENTMODE      = -19
KP_ERR_CORRUPT_DATA      = -23
KP_ERR_NO_KEY_ON_PILOT   = -24
KP_ERR_TO_FEW_DATA       = -26

# LED Color Codes
KP_LED_COLOR_GREEN       = 0
KP_LED_COLOR_RED         = 1
KP_LED_COLOR_OFF         = 2

# USB Device Datenstruktur
class KEYPILOT_USB_DEVICE(Structure):
	_fields_ = [
	("serialNumber",  c_char * 50),
	("connectionStatus" , c_int),
	("eventMode"        , c_int),
	("vendorName"       , c_char * 100),
	("vendorID"         , c_int),
	("productID"        , c_int),
	("version"          , c_int),
	("friendlyName"     , c_char * 255),
	("description"      , c_char * 255),
	("prName"           , c_wchar * 128),
	("deviceID"         , c_int) ]

# KeyData Struktur
class KEYPILOT_KEYDATA(Structure):
	_fields_ = [
	("validData",c_int),
	("command",c_char * 12),
	("keyID",c_char * 13),
	("famCode",c_char * 3),
	("crc",c_char * 3)]

# Callback Funktion fuer den Event Mode
NOTICECBFUNC   = WINFUNCTYPE(c_int,c_int,POINTER(KEYPILOT_KEYDATA))
DISCONNECTFUNC = WINFUNCTYPE(c_int,c_int)

def KeyPilotNotice(c_int,KeyData):
	kd = KeyData[0]
	if kd.validData == 1:
		print "Key empfangen: ", kd.keyID
	else:
		print "Key entfernt"
	return KP_OK

def KeyPilotDisconnect(devID):
	print "disconnect from: ", devID
	kp.kpUSBDisconnectFromDevice(devID)
	sys.exit(0)
	return KP_OK

kpNoticeCallback = NOTICECBFUNC(KeyPilotNotice)
kpDisconnectCallback = DISCONNECTFUNC(KeyPilotDisconnect)

key = KEYPILOT_KEYDATA()

# DLL laden
kp = ctypes.windll.LoadLibrary("KeyPilotUSB")


# Operationen auf dem Key Pilot
kp.kpUSBScanDevices()
count = kp.kpUSBGetNumberOfDevices()

if count <= 0:
	print "keine KeyPilots gefunden"
	sys.exit(0)

print "anzahl Keypilots: ", count

st = KEYPILOT_USB_DEVICE * count
stt = st()

ret = kp.kpUSBGetDeviceList(byref(stt),count)

if ret != KP_OK:
	print "Device Liste konnte nicht geholt werden"
	sys.exit(0)

print "-------------"
print "gefundene Geraete:"
for i in range(0,count):
	print "device ID: ",stt[i].deviceID
	print "seriennummer: ",stt[i].serialNumber
print "------------------"

dev = stt[0].deviceID

ret = kp.kpUSBConnectToDevice(dev)

if ret != KP_OK:
	print "Verbindung zum KeyPilot konnte nicht hergestellt werden"
	sys.exit(0)

ret = kp.kpUSBGetKeyID(dev,byref(key))

if ret != KP_OK:
	print "konnte Schluessel nicht lesen"
	sys.exit(0)

# Augabe der erhaltenen Daten
if key.validData != 0:
	print "key ID: ", key.keyID
else:
	print "Es liegt kein Schluessel auf"

errStrType = c_char * 100
errStr = errStrType()
kp.kpUSBGetErrorString(KP_ERR_NO_DEVICE_FOUND,errStr,100)
print "Sample Error String: ", errStr.value

pageSize = kp.kpUSBGetKeyPageSize(dev)
print "Page size: ", pageSize

bufferArray = c_char * pageSize
buffer = bufferArray()

# seite 0 vom schluessel einlesen und in hex ausgeben
print "lese Page 0 vom Schluessel"
ret = kp.kpUSBReadPageFromKey(dev,0,buffer,pageSize)
if ret != 0:
	print "Read error"
	sys.exit(0)
else:
	print "Gelesene seite:"
	for i in range(0,sizeof(buffer)):
		print buffer[i].encode('hex').upper(),
	print

# Event Mode
kp.kpUSBSetEventmode(dev,1,1,kpNoticeCallback,kpDisconnectCallback)

while(True):
	time.sleep(100)

kp.kpUSBDisconnectFromDevice(dev)
